@@ -1,5 +1,5 @@
 /* GNU ddrescue - Data recovery tool
-   Copyright (C) 2004-2024 Antonio Diaz Diaz.
+   Copyright (C) 2004-2025 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include <algorithm>
 #include <cctype>
 #include <cerrno>
-#include <climits>
+#include <climits>		// CHAR_BIT, SSIZE_MAX
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
@@ -34,7 +34,7 @@
 #include <string>
 #include <vector>
 #include <fcntl.h>
-#include <stdint.h>
+#include <stdint.h>		// SIZE_MAX
 #include <unistd.h>
 #include <sys/stat.h>
 
@@ -134,6 +134,7 @@ void show_help( const int cluster, const int hardbs )
                "  -Z, --max-read-rate=<bytes>    maximum read rate in bytes/s\n"
                "      --ask                      ask for confirmation before starting the copy\n"
                "      --command-mode             execute commands from standard input\n"
+               "      --continue-on-errno=<n>[,<n>]  treat errno code <n> as non-fatal\n"
                "      --cpass=<range>            select what copying pass(es) to run\n"
                "      --delay-slow=<interval>    initial delay before checking slow reads [30]\n"
                "      --log-events=<file>        log significant events in <file>\n"
@@ -167,8 +168,7 @@ namespace {
    Where the optional "unit" is one of 's', 'm', 'h', or 'd'.
    Return the number of seconds, or exit with status 1 if error.
 */
-Rational parse_rational_time( const char * const arg,
-                              const char * const option_name,
+Rational parse_rational_time( const char * const arg, const char * const pn,
                               const bool comma = false, const int max_den = 100 )
   {
   Rational r;
@@ -185,15 +185,14 @@ Rational parse_rational_time( const char * const arg,
       case  0 : break;
       case ',': if( comma ) break;			// fall through
       default: show_option_error( arg, "Bad unit in time interval argument of",
-                                  option_name ); std::exit( 1 );
+                                  pn ); std::exit( 1 );
       }
     if( !comma && arg[c] != 0 && arg[c] != ',' && arg[c+1] == ',' )
-      { show_option_error( arg, "Extra characters in argument of",
-                           option_name ); std::exit( 1 ); }
+      { show_option_error( arg, "Extra characters in argument of", pn );
+        std::exit( 1 ); }
     if( !r.error() && r >= 0 && r.denominator() <= max_den ) return r;
     }
-  show_option_error( arg, "Bad value in time interval argument of",
-                     option_name );
+  show_option_error( arg, "Bad value in time interval argument of", pn );
   std::exit( 1 );
   }
 
@@ -208,11 +207,11 @@ bool check_identical( const char * const iname, const char * const oname,
   {
   struct stat istat, ostat, mapstat;
   bool iexists = false, oexists = false, mapexists = false;
-  bool same = ( std::strcmp( iname, oname ) == 0 );
+  bool same = std::strcmp( iname, oname ) == 0;
   if( !same )
     {
-    iexists = ( stat( iname, &istat ) == 0 );
-    oexists = ( stat( oname, &ostat ) == 0 );
+    iexists = stat( iname, &istat ) == 0;
+    oexists = stat( oname, &ostat ) == 0;
     if( iexists && oexists && istat.st_ino == ostat.st_ino &&
         istat.st_dev == ostat.st_dev ) same = true;
     }
@@ -220,10 +219,10 @@ bool check_identical( const char * const iname, const char * const oname,
     { show_file_error( oname, "Infile and outfile are the same." ); return true; }
   if( mapname )
     {
-    same = ( std::strcmp( iname, mapname ) == 0 );
+    same = std::strcmp( iname, mapname ) == 0;
     if( !same )
       {
-      mapexists = ( stat( mapname, &mapstat ) == 0 );
+      mapexists = stat( mapname, &mapstat ) == 0;
       if( iexists && mapexists && istat.st_ino == mapstat.st_ino &&
           istat.st_dev == mapstat.st_dev ) same = true;
       }
@@ -410,9 +409,10 @@ void device_id_and_size( const long long size, const int fd,
   }
 
 
-void about_to_copy( const Rescuebook & rescuebook, const char * const iname,
+bool about_to_copy( const Rescuebook & rescuebook, const char * const iname,
                     const char * const oname, const long long insize,
-                    const int ides, const bool ask )
+                    const int cluster, const int ides, const int o_direct_out,
+                    const int o_trunc, const bool ask )
   {
   if( ask || verbosity >= 0 )
     std::printf( "%s %s\n", Program_name, PROGVERSION );
@@ -436,127 +436,6 @@ void about_to_copy( const Rescuebook & rescuebook, const char * const iname,
                    format_num( rescuebook.domain().in_size() ),
                  c, iname, iid.c_str(), p, oname, oid.c_str() );
     }
-  }
-
-
-bool user_agrees_ids( const Rescuebook & rescuebook, const char * const iname,
-                      const char * const oname, const long long insize,
-                      const int ides )
-  {
-  about_to_copy( rescuebook, iname, oname, insize, ides, true );
-  std::fputs( "Proceed (y/N)? ", stdout );
-  return safe_fflush( stdout ) && std::tolower( std::fgetc( stdin ) ) == 'y';
-  }
-
-
-long long adjusted_insize( const int ides, const Domain * const test_domain )
-  {
-  long long insize = lseek( ides, 0, SEEK_END );
-  if( insize >= 0 && test_domain )
-    {
-    const long long size = test_domain->end();
-    if( insize <= 0 || insize > size ) insize = size;
-    }
-  return insize;
-  }
-
-
-int do_rescue( const long long offset, Domain & domain,
-               const Domain * const test_domain, const Mb_options & mb_opts,
-               const Rb_options & rb_opts, const char * const iname,
-               const char * const oname, const char * const mapname,
-               const int cluster, const int hardbs, const int o_direct_out,
-               const int o_trunc, const bool ask, const bool command_mode,
-               const bool preallocate, const bool synchronous,
-               const bool check_input_size )
-  {
-  if( rb_opts.same_file && o_trunc )
-    {
-    show_error( "Option '--same-file' is incompatible with '--truncate'.", 0, true );
-    return 1;
-    }
-
-  // use same flags as reopen_infile
-  const int ides = open( iname, O_RDONLY | rb_opts.o_direct_in | O_BINARY );
-  if( ides < 0 )
-    { show_file_error( iname, "Can't open input file", errno ); return 1; }
-  const long long insize = adjusted_insize( ides, test_domain );
-  if( insize < 0 )
-    { show_file_error( iname, "Input file is not seekable." ); return 1; }
-
-  Rescuebook rescuebook( offset, insize, domain, test_domain, mb_opts, rb_opts,
-                         iname, oname, mapname, cluster, hardbs, synchronous );
-
-  if( check_input_size )
-    {
-    if( !rescuebook.mapfile_exists() || insize <= 0 ||
-        rescuebook.mapfile_insize() <= 0 ||
-        rescuebook.mapfile_insize() >= LLONG_MAX )
-      {
-      show_error( "Can't check input file size.\n"
-                  "          Mapfile is unfinished or missing, or size is invalid." );
-      return 1;
-      }
-    if( rescuebook.mapfile_insize() != insize )
-      {
-      show_error( "Input file size differs from size calculated from mapfile." );
-      return 1;
-      }
-    }
-  if( rescuebook.domain().empty() )
-    {
-    if( rescuebook.complete_only && !rescuebook.mapfile_exists() )
-      { show_error( "Nothing to complete; mapfile is missing or empty.", 0, true );
-        return 1; }
-    return empty_domain();
-    }
-  if( o_trunc && !rescuebook.blank() )
-    {
-    show_error( "Outfile truncation and mapfile input are incompatible.", 0, true );
-    return 1;
-    }
-  if( rescuebook.read_only() ) return not_writable( rescuebook.pname( false ) );
-
-  if( ask && !user_agrees_ids( rescuebook, iname, oname, insize, ides ) )
-    return 1;
-
-  const int odes =
-    open( oname, ( rescuebook.compare_before_write ? O_RDWR : O_WRONLY ) |
-                 O_CREAT | o_direct_out | o_trunc | O_BINARY, outmode );
-  if( odes < 0 )
-    { show_file_error( oname, "Can't open output file", errno ); return 1; }
-  if( lseek( odes, 0, SEEK_SET ) )
-    { show_file_error( oname, "Output file is not seekable." ); return 1; }
-  if( preallocate && lseek( odes, 0, SEEK_END ) - rescuebook.offset() <
-                     rescuebook.domain().end() )
-    {
-#if defined _POSIX_ADVISORY_INFO && _POSIX_ADVISORY_INFO > 0
-    if( posix_fallocate( odes, rescuebook.domain().pos() + rescuebook.offset(),
-                         rescuebook.domain().size() ) != 0 )
-      { show_file_error( oname, "Can't preallocate output file", errno );
-        return 1; }
-#else
-    show_file_error( oname, "warning: preallocation not available." );
-#endif
-    }
-
-  if( rescuebook.filename() && !rescuebook.mapfile_exists() &&
-      !rescuebook.write_mapfile( 0, true ) )
-    { show_file_error( mapname, "Can't create mapfile", errno ); return 1; }
-
-  if( command_mode ) return rescuebook.do_commands( ides, odes );
-
-  if( !event_logger.open_file() )
-    { show_file_error( event_logger.filename(),
-                       "Can't open file for logging events", errno ); return 1; }
-  if( !rate_logger.open_file() )
-    { show_file_error( rate_logger.filename(),
-                       "Can't open file for logging rates", errno ); return 1; }
-  if( !read_logger.open_file() )
-    { show_file_error( read_logger.filename(),
-                       "Can't open file for logging reads", errno ); return 1; }
-
-  if( !ask ) about_to_copy( rescuebook, iname, oname, insize, ides, false );
   if( verbosity >= 1 )
     {
     std::printf( "    Starting positions: infile = %sB,  outfile = %sB\n",
@@ -565,10 +444,10 @@ int do_rescue( const long long offset, Domain & domain,
     std::printf( "    Copy block size: %3d sectors", cluster );
     if( rescuebook.skipbs > 0 )
       std::printf( "       Initial skip size: %lld sectors\n",
-                   rescuebook.skipbs / hardbs );
+                   rescuebook.skipbs / rescuebook.hardbs() );
     else
       std::fputs( "       Skipping disabled\n", stdout );
-    std::printf( "Sector size: %sBytes\n", format_num( hardbs, 99999 ) );
+    std::printf( "Sector size: %sBytes\n", format_num( rescuebook.hardbs(), 99999 ) );
     if( verbosity >= 2 )
       {
       bool nl = false;
@@ -620,13 +499,123 @@ int do_rescue( const long long offset, Domain & domain,
         { nl = true; std::fputs( "Reverse mode", stdout ); }
       if( nl ) { nl = false; std::fputc( '\n', stdout ); }
       }
-    std::fputc( '\n', stdout );
     }
+  if( ask )
+    {
+    std::fputs( "Proceed (y/N)? ", stdout );
+    if( !safe_fflush( stdout ) || std::tolower( std::fgetc( stdin ) ) != 'y' )
+      return false;
+    }
+  if( ask || verbosity >= 1 ) std::fputc( '\n', stdout );
+  return true;
+  }
+
+
+long long adjusted_insize( const int ides, const Domain * const test_domain )
+  {
+  long long insize = lseek( ides, 0, SEEK_END );
+  if( insize >= 0 && test_domain )
+    {
+    const long long size = test_domain->end();
+    if( insize <= 0 || insize > size ) insize = size;
+    }
+  return insize;
+  }
+
+
+int do_rescue( const long long offset, Domain & domain,
+               const Domain * const test_domain, const Mb_options & mb_opts,
+               const Rb_options & rb_opts, const char * const iname,
+               const char * const oname, const char * const mapname,
+               const int cluster, const int hardbs, const int o_direct_out,
+               const int o_trunc, const bool ask, const bool command_mode,
+               const bool preallocate, const bool synchronous,
+               const bool check_input_size )
+  {
+  if( rb_opts.same_file && o_trunc )
+    {
+    show_error( "Option '--same-file' is incompatible with '--truncate'.", 0, true );
+    return 1;
+    }
+
+  // use same flags as reopen_infile
+  const int ides = open( iname, O_RDONLY | rb_opts.o_direct_in | O_BINARY );
+  if( ides < 0 )
+    { show_file_error( iname, "Can't open input file", errno ); return 1; }
+  const long long insize = adjusted_insize( ides, test_domain );
+  if( insize < 0 )
+    { show_file_error( iname, "Input file is not seekable." ); return 1; }
+
+  Rescuebook rescuebook( offset, insize, domain, test_domain, mb_opts, rb_opts,
+                         iname, oname, mapname, cluster, hardbs, synchronous );
+
+  if( check_input_size )
+    {
+    if( !rescuebook.mapfile_exists() || insize <= 0 ||
+        rescuebook.mapfile_insize() <= 0 ||
+        rescuebook.mapfile_insize() >= LLONG_MAX )
+      { show_error( "Can't check input file size.\n          Mapfile is "
+                    "unfinished or missing, or size is invalid." ); return 1; }
+    if( rescuebook.mapfile_insize() != insize )
+      { show_error( "Input file size differs from size calculated from mapfile." );
+        return 1; }
+    }
+  if( rescuebook.domain().empty() )
+    {
+    if( rescuebook.complete_only && !rescuebook.mapfile_exists() )
+      { show_error( "Nothing to complete; mapfile is missing or empty.", 0,
+                    true ); return 1; }
+    return empty_domain();
+    }
+  if( o_trunc && !rescuebook.blank() )
+    { show_error( "Outfile truncation and mapfile input are incompatible.", 0,
+                  true ); return 1; }
+  if( rescuebook.read_only() ) return not_writable( rescuebook.pname( false ) );
+
+  if( !about_to_copy( rescuebook, iname, oname, insize, cluster, ides,
+                      o_direct_out, o_trunc, ask ) ) return 1;
+
+  const int odes =
+    open( oname, ( rescuebook.compare_before_write ? O_RDWR : O_WRONLY ) |
+                 O_CREAT | o_direct_out | o_trunc | O_BINARY, outmode );
+  if( odes < 0 )
+    { show_file_error( oname, "Can't open output file", errno ); return 1; }
+  if( lseek( odes, 0, SEEK_SET ) )
+    { show_file_error( oname, "Output file is not seekable." ); return 1; }
+  if( preallocate && lseek( odes, 0, SEEK_END ) - rescuebook.offset() <
+                     rescuebook.domain().end() )
+    {
+#if defined _POSIX_ADVISORY_INFO && _POSIX_ADVISORY_INFO > 0
+    if( posix_fallocate( odes, rescuebook.domain().pos() + rescuebook.offset(),
+                         rescuebook.domain().size() ) != 0 )
+      { show_file_error( oname, "Can't preallocate output file", errno );
+        return 1; }
+#else
+    show_file_error( oname, "warning: preallocation not available." );
+#endif
+    }
+
+  if( rescuebook.filename() && !rescuebook.mapfile_exists() &&
+      !rescuebook.write_mapfile( 0, true ) )
+    { show_file_error( mapname, "Can't create mapfile", errno ); return 1; }
+
+  if( command_mode ) return rescuebook.do_commands( ides, odes );
+
+  if( !event_logger.open_file() )
+    { show_file_error( event_logger.filename(),
+                       "Can't open file for logging events", errno ); return 1; }
+  if( !rate_logger.open_file() )
+    { show_file_error( rate_logger.filename(),
+                       "Can't open file for logging rates", errno ); return 1; }
+  if( !read_logger.open_file() )
+    { show_file_error( read_logger.filename(),
+                       "Can't open file for logging reads", errno ); return 1; }
+
   return rescuebook.do_rescue( ides, odes );
   }
 
 
-void parse_cpass( const char * const arg, const char * const option_name,
+void parse_cpass( const char * const arg, const char * const pn,
                   Rb_options & rb_opts )
   {
   const char * p = arg;
@@ -648,7 +637,7 @@ void parse_cpass( const char * const arg, const char * const option_name,
     if( *p == 0 ) return;
     if( *p == ',' ) ++p; else break;
     }
-  show_option_error( arg, "Invalid pass or range of passes in", option_name );
+  show_option_error( arg, "Invalid pass or range of passes in", pn );
   std::exit( 1 );
   }
 
@@ -675,14 +664,30 @@ void parse_mapfile_intervals( const char * const arg, const char * const pn,
   }
 
 
-void parse_pause_on_error( const char * const p, const char * const pn,
+void parse_errno_vector( const char * const arg, const char * const pn,
+                         Rb_options & rb_opts )
+  {
+  const char * p = arg;
+  while( std::isdigit( *(const unsigned char *)p ) )
+    {
+    const char * tail;
+    rb_opts.errno_vector.push_back( getnum( p, pn, 0, 1, INT_MAX, &tail ) );
+    if( *tail == 0 ) return;
+    if( *tail == ',' ) p = tail + 1; else break;
+    }
+  show_option_error( arg, "Invalid errno value in", pn );
+  std::exit( 1 );
+  }
+
+
+void parse_pause_on_error( const char * const arg, const char * const pn,
                            Rb_options & rb_opts )
   {
-  rb_opts.simulated_poe = ( p[0] == 's' );
+  rb_opts.simulated_poe = *arg == 's';
   if( rb_opts.simulated_poe )
-    rb_opts.pause_on_error = parse_rational_time( p + 1, pn );
+    rb_opts.pause_on_error = parse_rational_time( arg + 1, pn );
   else
-    rb_opts.pause_on_error = parse_time_interval( p, pn );
+    rb_opts.pause_on_error = parse_time_interval( arg, pn );
   }
 
 
@@ -691,17 +696,17 @@ void parse_skipbs( const char * const arg, const char * const pn,
   {
   const char * tail = arg;
 
-  if( tail[0] != ',' )
+  if( *tail != ',' )
     rb_opts.skipbs = getnum( arg, pn, hardbs, 0, rb_opts.max_max_skipbs, &tail );
-  if( tail[0] == ',' )
+  if( *tail == ',' )
     {
     rb_opts.max_skipbs = getnum( tail + 1, pn, hardbs, Rb_options::min_skipbs,
                                  rb_opts.max_max_skipbs, &tail );
-    if( tail[0] )
+    if( *tail )
       { show_option_error( arg, "Extra characters in argument of", pn );
         std::exit( 1 ); }
     }
-  else if( tail[0] )
+  else if( *tail )
     { show_option_error( arg, "Bad separator in argument of", pn );
       std::exit( 1 ); }
   if( rb_opts.skipbs > 0 && rb_opts.skipbs < Rb_options::min_skipbs )
@@ -722,6 +727,17 @@ void check_o_direct()
 } // end namespace
 
 
+void show_final_msg( const char * const msg, const int errcode )
+  {
+  if( verbosity < 0 ) return;
+  if( errcode > 0 )
+    std::fprintf( stderr, "%s: %s: %s (errno=%u)\n", program_name, msg,
+                  std::strerror( errcode ), errcode );
+  else
+    std::fprintf( stderr, "%s: %s\n", program_name, msg );
+  }
+
+
 bool Rescuebook::reopen_infile()
   {
   if( ides_ >= 0 ) close( ides_ );
@@ -766,71 +782,72 @@ int main( const int argc, const char * const argv[] )
   for( int i = 1; i < argc; ++i )
     { command_line += ' '; command_line += argv[i]; }
 
-  enum { opt_ask = 256, opt_cm, opt_cpa, opt_ds, opt_eve, opt_mi,
+  enum { opt_ask = 256, opt_cm, opt_con, opt_cpa, opt_ds, opt_eve, opt_mi,
          opt_msr, opt_poe, opt_pop, opt_rat, opt_rea, opt_rs, opt_sf };
   const Arg_parser::Option options[] =
     {
-    { 'a', "min-read-rate",        Arg_parser::yes },
-    { 'A', "try-again",            Arg_parser::no  },
-    { 'b', "sector-size",          Arg_parser::yes },
-    { 'B', "binary-prefixes",      Arg_parser::no  },
-    { 'c', "cluster-size",         Arg_parser::yes },
-    { 'C', "complete-only",        Arg_parser::no  },
-    { 'd', "idirect",              Arg_parser::no  },
-    { 'D', "odirect",              Arg_parser::no  },
-    { 'e', "max-bad-areas",        Arg_parser::yes },
-    { 'e', "max-errors",           Arg_parser::yes },
-    { 'E', "max-error-rate",       Arg_parser::yes },
-    { 'f', "force",                Arg_parser::no  },
-    { 'F', "fill-mode",            Arg_parser::yes },
-    { 'G', "generate-mode",        Arg_parser::no  },
-    { 'h', "help",                 Arg_parser::no  },
-    { 'H', "test-mode",            Arg_parser::yes },
-    { 'i', "input-position",       Arg_parser::yes },
-    { 'I', "check-input-size",     Arg_parser::no  },
-    { 'I', "verify-input-size",    Arg_parser::no  },
-    { 'J', "check-on-error",       Arg_parser::no  },
-    { 'J', "verify-on-error",      Arg_parser::no  },
-    { 'K', "skip-size",            Arg_parser::yes },
-    { 'L', "loose-domain",         Arg_parser::no  },
-    { 'm', "domain-mapfile",       Arg_parser::yes },
-    { 'M', "retrim",               Arg_parser::no  },
-    { 'n', "no-scrape",            Arg_parser::no  },
-    { 'N', "no-trim",              Arg_parser::no  },
-    { 'o', "output-position",      Arg_parser::yes },
-    { 'O', "reopen-on-error",      Arg_parser::no  },
-    { 'p', "preallocate",          Arg_parser::no  },
-    { 'P', "data-preview",         Arg_parser::maybe },
-    { 'q', "quiet",                Arg_parser::no  },
-    { 'r', "retry-passes",         Arg_parser::yes },
-    { 'R', "reverse",              Arg_parser::no  },
-    { 's', "size",                 Arg_parser::yes },
-    { 'S', "sparse",               Arg_parser::no  },
-    { 't', "truncate",             Arg_parser::no  },
-    { 'T', "timeout",              Arg_parser::yes },
-    { 'u', "unidirectional",       Arg_parser::no  },
-    { 'v', "verbose",              Arg_parser::no  },
-    { 'V', "version",              Arg_parser::no  },
-    { 'w', "ignore-write-errors",  Arg_parser::no  },
-    { 'W', "compare-before-write", Arg_parser::no  },
-    { 'x', "extend-outfile",       Arg_parser::yes },
-    { 'X', "max-read-errors",      Arg_parser::yes },
-    { 'y', "synchronous",          Arg_parser::no  },
-    { 'Z', "max-read-rate",        Arg_parser::yes },
-    { opt_ask, "ask",              Arg_parser::no  },
-    { opt_cm,  "command-mode",     Arg_parser::no  },
-    { opt_cpa, "cpass",            Arg_parser::yes },
-    { opt_ds,  "delay-slow",       Arg_parser::yes },
-    { opt_eve, "log-events",       Arg_parser::yes },
-    { opt_mi,  "mapfile-interval", Arg_parser::yes },
-    { opt_msr, "max-slow-reads",   Arg_parser::yes },
-    { opt_poe, "pause-on-error",   Arg_parser::yes },
-    { opt_pop, "pause-on-pass",    Arg_parser::yes },
-    { opt_rat, "log-rates",        Arg_parser::yes },
-    { opt_rea, "log-reads",        Arg_parser::yes },
-    { opt_rs,  "reset-slow",       Arg_parser::no  },
-    { opt_sf,  "same-file",        Arg_parser::no  },
-    {  0 , 0,                      Arg_parser::no  } };
+    { 'a', "min-read-rate",         Arg_parser::yes },
+    { 'A', "try-again",             Arg_parser::no  },
+    { 'b', "sector-size",           Arg_parser::yes },
+    { 'B', "binary-prefixes",       Arg_parser::no  },
+    { 'c', "cluster-size",          Arg_parser::yes },
+    { 'C', "complete-only",         Arg_parser::no  },
+    { 'd', "idirect",               Arg_parser::no  },
+    { 'D', "odirect",               Arg_parser::no  },
+    { 'e', "max-bad-areas",         Arg_parser::yes },
+    { 'e', "max-errors",            Arg_parser::yes },
+    { 'E', "max-error-rate",        Arg_parser::yes },
+    { 'f', "force",                 Arg_parser::no  },
+    { 'F', "fill-mode",             Arg_parser::yes },
+    { 'G', "generate-mode",         Arg_parser::no  },
+    { 'h', "help",                  Arg_parser::no  },
+    { 'H', "test-mode",             Arg_parser::yes },
+    { 'i', "input-position",        Arg_parser::yes },
+    { 'I', "check-input-size",      Arg_parser::no  },
+    { 'I', "verify-input-size",     Arg_parser::no  },
+    { 'J', "check-on-error",        Arg_parser::no  },
+    { 'J', "verify-on-error",       Arg_parser::no  },
+    { 'K', "skip-size",             Arg_parser::yes },
+    { 'L', "loose-domain",          Arg_parser::no  },
+    { 'm', "domain-mapfile",        Arg_parser::yes },
+    { 'M', "retrim",                Arg_parser::no  },
+    { 'n', "no-scrape",             Arg_parser::no  },
+    { 'N', "no-trim",               Arg_parser::no  },
+    { 'o', "output-position",       Arg_parser::yes },
+    { 'O', "reopen-on-error",       Arg_parser::no  },
+    { 'p', "preallocate",           Arg_parser::no  },
+    { 'P', "data-preview",          Arg_parser::maybe },
+    { 'q', "quiet",                 Arg_parser::no  },
+    { 'r', "retry-passes",          Arg_parser::yes },
+    { 'R', "reverse",               Arg_parser::no  },
+    { 's', "size",                  Arg_parser::yes },
+    { 'S', "sparse",                Arg_parser::no  },
+    { 't', "truncate",              Arg_parser::no  },
+    { 'T', "timeout",               Arg_parser::yes },
+    { 'u', "unidirectional",        Arg_parser::no  },
+    { 'v', "verbose",               Arg_parser::no  },
+    { 'V', "version",               Arg_parser::no  },
+    { 'w', "ignore-write-errors",   Arg_parser::no  },
+    { 'W', "compare-before-write",  Arg_parser::no  },
+    { 'x', "extend-outfile",        Arg_parser::yes },
+    { 'X', "max-read-errors",       Arg_parser::yes },
+    { 'y', "synchronous",           Arg_parser::no  },
+    { 'Z', "max-read-rate",         Arg_parser::yes },
+    { opt_ask, "ask",               Arg_parser::no  },
+    { opt_cm,  "command-mode",      Arg_parser::no  },
+    { opt_con, "continue-on-errno", Arg_parser::yes },
+    { opt_cpa, "cpass",             Arg_parser::yes },
+    { opt_ds,  "delay-slow",        Arg_parser::yes },
+    { opt_eve, "log-events",        Arg_parser::yes },
+    { opt_mi,  "mapfile-interval",  Arg_parser::yes },
+    { opt_msr, "max-slow-reads",    Arg_parser::yes },
+    { opt_poe, "pause-on-error",    Arg_parser::yes },
+    { opt_pop, "pause-on-pass",     Arg_parser::yes },
+    { opt_rat, "log-rates",         Arg_parser::yes },
+    { opt_rea, "log-reads",         Arg_parser::yes },
+    { opt_rs,  "reset-slow",        Arg_parser::no  },
+    { opt_sf,  "same-file",         Arg_parser::no  },
+    { 0, 0,                         Arg_parser::no  } };
 
   const Arg_parser parser( argc, argv, options );
   if( parser.error().size() )				// bad option
@@ -854,7 +871,7 @@ int main( const int argc, const char * const argv[] )
       case 'C': rb_opts.complete_only = true; break;
       case 'd': rb_opts.o_direct_in = O_DIRECT; check_o_direct(); break;
       case 'D': o_direct_out = O_DIRECT; check_o_direct(); break;
-      case 'e': rb_opts.new_bad_areas_only = ( arg[0] == '+' );
+      case 'e': rb_opts.new_bad_areas_only = *arg == '+';
                 rb_opts.max_bad_areas = getnum( arg, pn, 0, 0, LONG_MAX ); break;
       case 'E': rb_opts.max_error_rate = getnum( arg, pn, hardbs, 0 ); break;
       case 'f': force = true; break;
@@ -877,8 +894,8 @@ int main( const int argc, const char * const argv[] )
       case 'o': opos = getnum( arg, pn, hardbs, 0 ); break;
       case 'O': rb_opts.reopen_on_error = true; break;
       case 'p': preallocate = true; break;
-      case 'P': rb_opts.preview_lines =
-                  arg[0] ? getnum( arg, pn, 0, 1, 32 ) : 3; break;
+      case 'P': rb_opts.preview_lines = *arg ? getnum( arg, pn, 0, 1, 32 ) : 3;
+                break;
       case 'q': verbosity = -1; break;
       case 'r': rb_opts.max_retries = getnum( arg, pn, 0, -1, INT_MAX / 2 ); break;
       case 'R': rb_opts.reverse = true; break;
@@ -897,6 +914,7 @@ int main( const int argc, const char * const argv[] )
       case 'Z': rb_opts.max_read_rate = getnum( arg, pn, hardbs, 1 ); break;
       case opt_ask: ask = true; break;
       case opt_cm:  set_mode( program_mode, m_command ); break;
+      case opt_con: parse_errno_vector( arg, pn, rb_opts ); break;
       case opt_cpa: parse_cpass( arg, pn, rb_opts ); break;
       case opt_ds:  rb_opts.delay_slow = parse_time_interval( arg, pn ); break;
       case opt_eve: event_logger.set_filename( arg ); break;
@@ -915,7 +933,7 @@ int main( const int argc, const char * const argv[] )
 
   if( opos < 0 ) opos = ipos;
   if( hardbs < 1 ) hardbs = default_hardbs;
-  if( cluster >= INT_MAX / hardbs ) cluster = ( INT_MAX / hardbs ) - 1;
+  if( cluster >= INT_MAX / hardbs ) cluster = INT_MAX / hardbs - 1;
   if( cluster < 1 ) cluster = cluster_bytes / hardbs;
   if( cluster < 1 ) cluster = 1;
 
